package main

import (
	"bot/api"
	"encoding/gob"
	"log"
	"os"
	"runtime/debug"
	"strconv"

	"github.com/boltdb/bolt"
	tgbotapi "github.com/go-telegram-bot-api/telegram-bot-api/v5"
	"gopkg.in/yaml.v2"
)

var filename = "bot.map"

type Config struct {
	Account struct {
		Mode     string `yaml:"mode"` // pooling or webhook
		Token    string `yaml:"token"`
		Owner    int64  `yaml:"owner"`
		Endpoint string `yaml:"endpoint"`
		Port     int    `yaml:"port"`
	} `yaml:"account"`
}

var BotConfig Config
var bucketname = []byte("msg2chatid")
var db *bolt.DB

func main() {
	yamlFile, err := os.ReadFile("bot.yaml")
	if err != nil {
		panic("read yaml fail: " + err.Error())
	}
	err = yaml.Unmarshal(yamlFile, &BotConfig)
	if err != nil {
		panic("unmarshal yaml fail: " + err.Error())
	}
	db, err = bolt.Open("bot.db", 0600, nil)
	if err != nil {
		panic("open db fail: " + err.Error())
	}
	defer db.Close()
	db.Update(func(tx *bolt.Tx) error {
		_, err := tx.CreateBucketIfNotExists([]byte(bucketname))
		if err != nil {
			panic("create bucket fail: " + err.Error())
		}
		return nil
	})
	api.InitBot(BotConfig.Account.Mode, BotConfig.Account.Token, BotConfig.Account.Endpoint, BotConfig.Account.Port, handleUpdate)
}

func deliverIncomingMsg(msg api.SimpleMsg) {
	log.Printf("receive message from %d %s\n", msg.ChatId, msg.Name)
	msgid := api.ForwardMsg(BotConfig.Account.Owner, msg.ChatId, msg.MessageID)
	db.Update(func(tx *bolt.Tx) error {
		b := tx.Bucket(bucketname)
		b.Put([]byte(strconv.Itoa(msgid)), []byte(strconv.Itoa(int(msg.ChatId))))
		log.Printf("store chatid %d for message %d\n", msg.ChatId, msgid)
		return nil
	})
	log.Printf("forward message from %d, message id %d\n", msg.ChatId, msgid)
}

func directmsg(msg api.SimpleMsg) {
	chatid := int(0)
	for i := 1; i < len(msg.Text); i++ {
		if msg.Text[i] == ' ' {
			chatid, _ = strconv.Atoi(msg.Text[1:i])
			msg.Text = msg.Text[i+1:]
			break
		}
	}
	if chatid == 0 {
		api.SendMsg(msg.ChatId, "format invaild")
		return
	}
	if msg.Text != "" {
		api.SendMsg(int64(chatid), msg.Text)
	}
}

func deliverOutgoingMsg(msg api.SimpleMsg) {
	if msg.Text != "" && msg.Text[0] == '*' {
		directmsg(msg)
		return
	}
	var storechatid int
	db.View(func(tx *bolt.Tx) error {
		b := tx.Bucket(bucketname)
		v := b.Get([]byte(strconv.Itoa(int(msg.ReplyID))))
		if v != nil {
			log.Printf("found chatid %s for message %d\n", string(v), msg.ReplyID)
			storechatid, _ = strconv.Atoi(string(v))
		} else {
			storechatid = 0
		}
		return nil
	})
	if storechatid == 0 {
		api.SendMsg(msg.ChatId, "reply to forward ...")
	} else {
		if msg.Text != "" {
			api.SendMsg(int64(storechatid), msg.Text)
		} else if msg.PhotoID != "" {
			api.SendExistingPhoto(int64(storechatid), msg.PhotoID)
		} else if msg.VideoID != "" {
			api.SendExistingVideo(int64(storechatid), msg.VideoID)
		}
	}
}

func commander(msg api.SimpleMsg) {
}

func handleUpdate(update tgbotapi.Update) {
	defer func() {
		if r := recover(); r != nil {
			log.Printf("Recovered from panic in handleUpdate: %v\n", r)
			debug.PrintStack()
		}
	}()

	msg := api.FormatMsg(update)
	if msg.Type != "private" {
		return
	}
	if msg.Text != "" {
		if msg.Text[0] == '/' {
			if msg.FromID == BotConfig.Account.Owner {
				commander(msg)
			}
			return
		}
	}

	if msg.FromID != BotConfig.Account.Owner {
		deliverIncomingMsg(msg)
	} else {
		deliverOutgoingMsg(msg)
	}
}

func SaveMapToDisk(m map[int]int64) error {
	file, err := os.Create(filename)
	if err != nil {
		return err
	}
	defer file.Close()

	encoder := gob.NewEncoder(file)
	err = encoder.Encode(m)
	if err != nil {
		return err
	}

	return nil
}

func LoadMapFromDisk() (map[int]int64, error) {
	file, err := os.Open(filename)
	if err != nil {
		return make(map[int]int64), err
	}
	defer file.Close()

	decoder := gob.NewDecoder(file)
	var m map[int]int64
	err = decoder.Decode(&m)
	if err != nil {
		return make(map[int]int64), err
	}
	return m, nil
}
